/* Dyadic grids in arbitrary dimension */
/* Last edited on 2002-04-27 15:47:16 by stolfi */

#include <stdlib.h>
#include <js.h>
#include <dygrid.h>

nat8 dyg_depth(dyg_node_index k)
{
  int r = 1;
  while (k > 1) { k = (k/2); r++; }
  return r;
}

dyg_node *dyg_new_node(dyg_node *p)
{
  dyg_node *r = (dyg_node*)notnull(malloc(sizeof(dyg_node)), "no mem for dyg_node");
  r->c0 = NULL;
  r->c1 = NULL;
  r->p = p;
  return r;
}

void dyg_free_node(dyg_node **pp)
{
  dyg_node *p = (*pp);
  assert(p->c0 == NULL, "node has c0");
  assert(p->c1 == NULL, "node has c1");
  free(p);
  (*pp) = NULL;
}

dyg_header *dyg_new_tree(nat8 d)
{
  dyg_header *h = (dyg_header*)notnull(malloc(sizeof(dyg_header)), "no mem for dyg_header");
  dyg_node *r = dyg_new_node(NULL);
  h->d = d;
  h->root = r;
  return h;
}
  
void dyg_split_node(dyg_node *p)
{
  dyg_node *c0 = dyg_new_node(p);
  dyg_node *c1 = dyg_new_node(p);
  assert(p->c0 == NULL, "node has c0");
  assert(p->c1 == NULL, "node has c1");
  p->c0 = c0;
  p->c1 = c1;
}
  
void dyg_unsplit_node(dyg_node *p)
{
  assert(p->c0 != NULL, "node has no c0");
  assert(p->c1 != NULL, "node has no c1");
  dyg_free_node(&(p->c0));
  dyg_free_node(&(p->c1));
}
  

