/* See dg_grid.h */
/* Last edited on 2005-06-05 15:40:53 by stolfi */

#include <affirm.h>
#include <bool.h>
#include <dg_grid.h>
#include <indexing.h>
#include <stdlib.h>
#include <assert.h>
  
box_axis_set_t dg_axis_set_complement(dg_dim_t d, dg_axis_set_t A)
  {  return set32_difference(set32_range(0,d-1), A); }

/* THE DYADIC MULTIGRID */

dg_axis_t dg_split_axis(dg_dim_t d, dg_rank_t r)
  { return r % d; }
  
#define dg_GRID_POS_BITS(d,r,j)  (((r)+(d)-1-(j))/(d))
  /* Number of bits needed to represent a grid position within level {r}
    along axis {j} of a {d}-dimensional multigrid. */

void dg_grid_size(dg_dim_t d, dg_rank_t r, dg_grid_size_t gsz[])
  { int i;
    for (i = 0; i < d; i++) { gsz[i] = (1LL << dg_GRID_POS_BITS(d,r,i)); }
  }

dg_grid_pos_t dg_max_grid_pos(dg_dim_t d, dg_rank_t r, dg_axis_t i)
  { return (1LL << dg_GRID_POS_BITS(d,r,i))-1; }

/* CELL INDICES */
  
dg_rank_t dg_cell_rank(dg_cell_index_t k)
  { int r = 0;
    while (k > (1LL << 16)) { k >>= 16; r+= 16; }
    while (k > (1LL << 4)) { k >>= 4; r+= 4; }
    while (k > 1) { k >>= 1; r+= 1; }
    return r;
  }

void dg_cell_position(dg_dim_t d, dg_cell_index_t k, dg_grid_pos_t pos[])
  { dg_cell_index_t t = ((1LL << dg_cell_rank(k)) >> 1);
    int ax;
    for (ax = 0; ax < d; ax++) { pos[ax] = 0; }
    ax = 0;
    while (t != 0)
      { pos[ax] <<= 1; if ((k & t) != 0) { pos[ax] |= 1;  }
        ax++; if (ax >= d) { ax = 0; }
        t >>= 1;
      }
  }
  
dg_cell_index_t dg_cell_from_position(dg_dim_t d, dg_rank_t r, dg_grid_pos_t pos[])
  { /* First cell of layer {r}: */
    dg_cell_index_t k = (1LL << r);
    /* Limit for bit values that can be added to {k}: */
    dg_cell_index_t tlim = (1LL << r);
    int ax;
    for (ax = 0; ax < d; ax++) 
      { dg_grid_pos_t posax = pos[ax]; 
        /* Add {posax} to {k}, starting at bit {ax} with stride {d}: */
        uint64_t t = 1LL << ((r + d - ax) % d);
        while ((posax != 0) && (t < tlim))
          { if (posax & 1) 
              { if ((k & t) != 0) 
                  { k &= ~t; posax += 2; }
                else
                  { k |= t; }
              }
            posax >>= 1; t <<= d;
          }
      }
    return k;
  }

dg_cell_index_t dg_cell_shift(dg_dim_t d, dg_cell_index_t k, dg_axis_t ax, dg_grid_pos_t dp)
  { dg_rank_t rank = dg_cell_rank(k);
    /* Limit for bit values that can be added to {k}: */
    dg_cell_index_t tlim = (1LL << rank);
    /* Add {dp} to {k}, starting at bit {ax} with stride {d}: */
    dg_cell_index_t t = 1LL << ((rank + d - ax) % d);
    while ((dp != 0) && (t < tlim))
      { if (dp & 1) 
          { if ((k & t) != 0) 
              { k &= ~t; dp = dp + 2; }
            else
              { k |= t; }
          }
        dp >>= 1; t <<= d;
      }
    return k;
  }
  
dg_cell_index_t dg_cell_translate(dg_dim_t d, dg_cell_index_t k, dg_grid_pos_t dp[])
  { dg_rank_t rank = dg_cell_rank(k);
    /* Limit for bit values that can be added to {k}: */
    dg_cell_index_t tlim = (1LL << rank);
    int ax;
    for (ax = 0; ax < d; ax++) 
      { dg_grid_pos_t dpax = dp[ax]; 
        /* Add {dpax} to {k}, starting at bit {ax} with stride {d}: */
        uint64_t t = 1LL << ((rank + d - ax) % d);
        while ((dpax != 0) && (t < tlim))
          { if (dpax & 1) 
              { if ((k & t) != 0) 
                  { k &= ~t; dpax += 2; }
                else
                  { k |= t; }
              }
            dpax >>= 1; t <<= d;
          }
      }
    return k;
  }

dg_cell_index_t dg_cell_index_add(dg_dim_t d, dg_cell_index_t ka, dg_cell_index_t kb)
  { dg_rank_t ra = dg_cell_rank(ka);
    dg_rank_t rb = dg_cell_rank(kb);
    affirm(ra == rb, "unequal ranks");
    /* Limit for bit values that can be added to {ka}: */
    dg_cell_index_t tlim = (1LL << ra);
    /* Adds the interleaved cell positions. */
    /* Variable {carry} contains the {d} carry bits, rotated. */
    uint64_t t = 1;
    int carry = 0, cd = (1 << d);
    /* Discard leading bit of {kb}: */
    kb -= tlim;
    while (((kb > 0) || (carry != 0)) && (t < tlim))
      { if (kb & carry & 1) 
          { carry |= cd; }
        else if ((kb | carry) & 1)
          { if ((ka & t) != 0) 
              { ka &= ~t; carry |= cd; }
            else
              { ka |= t; }
          }
        kb >>= 1; carry >>= 1; t <<= 1;
      }
    return ka;
  }

/* Sizes of canonical cells for various dimensions: */
static double dg_root_size[10] = 
  { 1.0,                 
    1.0, 0.7071067811865475,
    1.0, 0.7937005259840997, 0.6299605249474366,
    1.0, 0.8408964152537145, 0.7071067811865475, 0.5946035575013605 
  };

void dg_cell_box_root_relative(dg_dim_t d, dg_cell_index_t k, interval_t B[])
  { dg_grid_pos_t pos[dg_MAX_GRID_DIM];
    dg_cell_position(d, k, pos);
    
    dg_grid_size_t gsz[dg_MAX_GRID_DIM];
    dg_grid_size(d, dg_cell_rank(k), gsz);
    
    int i;
    for (i = 0; i < d; i++)
      { double scale = 1.0/((double)(gsz[i]));
        LO(B[i]) = scale*((double)pos[i]);
        HI(B[i]) = scale*((double)(pos[i] + 1));
      }
  }

void dg_cell_box_canonical(dg_dim_t d, dg_cell_index_t k, interval_t B[])
  { dg_cell_box_root_relative(d, k, B); 
    int i, j;
    for (i = 0, j = d*(d-1)/2; i < d; i++, j++)
      { double wd = dg_root_size[j];
        LO(B[i]) *= wd;
        HI(B[i]) *= wd;
      }
  }
  
/* CELL PACKS */

uint64_t dg_pack_slot_count(dg_dim_t d, dg_grid_size_t psz[])
  { uint64_t n = 1;
    int i;
    for (i = 0; i < d; i++) { n *= psz[i]; }
    return n;
  }

void dg_pack_cells
  ( dg_dim_t d, 
    dg_cell_index_t b,
    dg_grid_size_t psz[], 
    dg_cell_index_t k[]
  )
{ dg_grid_pos_t rpos[d]; /* Relative position of cell in pack. */
  /* Indexing steps: */
  ix_step_t pst[d];
  pst[0] = 1;
  { dg_axis_t i; for (i=1; i <d; i++) { pst[i] = pst[i-1]*psz[i-1]; } }
  ix_fill(d, rpos, 0);
  int nC = 0;
  do
    { k[nC] = dg_cell_translate(d, b, rpos);  nC++; }
  while (! ix_next_F(d, rpos, psz, pst, NULL));
}

dg_pack_slot_index_t dg_pack_find_cell
  ( dg_dim_t d, 
    dg_cell_index_t k, 
    dg_cell_index_t b,
    dg_grid_size_t psz[]
  )
  {
    dg_rank_t kr = dg_cell_rank(k);
    dg_rank_t br = dg_cell_rank(b);
    if (kr != br) { return dg_NOT_SLOT; }

    dg_grid_pos_t kp[dg_MAX_GRID_DIM];
    dg_cell_position(d, k, kp);
    
    dg_grid_pos_t bp[dg_MAX_GRID_DIM];
    dg_cell_position(d, b, bp);
 
    int i;
    int pkx = 1;
    dg_pack_slot_index_t ix = 0;
    for (i = 0; i < d; i++)
      { demand(psz[i] > 0, "bad pack size");
        dg_grid_pos_t szi = dg_max_grid_pos(d, br, i);
        /* Get relative position of {k} in pack along axis {i}: */
        dg_grid_pos_t ei = (kp[i] - bp[i] + szi) % szi;
        if (ei >= psz[i]) { return dg_NOT_SLOT; }
        ix += ei * pkx;
        pkx *= psz[i];
      }
    return pkx;
  }

bool_t dg_pack_intersects_cell
  ( dg_dim_t d, 
    dg_cell_index_t k, 
    dg_cell_index_t b,
    dg_grid_size_t psz[]
  )
  {
    dg_rank_t br = dg_cell_rank(b);
    dg_grid_pos_t bp[dg_MAX_GRID_DIM];
    dg_cell_position(d, b, bp);
    
    /* If {k} is deeper than {b}, get its ancestor of same rank: */
    dg_rank_t kr = dg_cell_rank(k);
    if (kr > br) { k = k >> (kr - br); kr = br; }
    /* Get posns of min and max descendants of {k} on level {Er}: */
    dg_grid_pos_t kp_lo[dg_MAX_GRID_DIM];
    dg_cell_position(d, k << (br - kr), kp_lo);
    dg_grid_pos_t kp_hi[dg_MAX_GRID_DIM];
    dg_cell_position(d, ((k + 1) << (br - kr)) - 1, kp_hi);

    /* Get grid sizes on level {br}: */
    dg_grid_size_t bsz[dg_MAX_GRID_DIM];
    dg_grid_size(d, br, bsz);
    
    int i;
    for (i = 0; i < d; i++)
      { demand(psz[i] > 0, "bad pack size");
        dg_grid_size_t bszi = bsz[i];
        /* Get rel position range {plop..phip} of pack in level {br}: */ 
        dg_grid_pos_t plop = bp[i];
        dg_grid_pos_t phip = (phip + psz[i] - 1) % bszi;
        /* Make sure {phip>=plop}: */
        if (phip < plop) { phip += bszi; }
        /* Get rel position range {klop..khip} of {k} in the same level: */
        dg_grid_pos_t klop = kp_lo[i];
        dg_grid_pos_t khip = kp_hi[i];
        assert(klop <= khip); /* A single cell can't wrap around. */
        /* Check whether the ranges intersect (with or without wrap-around): */
        if (klop < plop)
          { if ((khip < plop) && (phip < klop + bszi)) { return FALSE; } }
        else if (plop < klop)
          { if (klop > phip) { return FALSE; }}
      }
    return TRUE;
  }

void dg_pack_box_root_relative
  ( dg_dim_t d, 
    dg_cell_index_t b,
    dg_grid_size_t psz[], 
    interval_t B[]
  )
  { dg_cell_box_root_relative(d, b, B);
    int i;
    for (i = 0; i < d; i++)
      { demand(psz[i] > 0, "bad pack size");
        HI(B[i]) = HI(B[i]) + (psz[i]-1)*(HI(B[i]) - LO(B[i]));
      }
  }

/* PRINTOUT */

void dg_cell_print(FILE *wr, dg_dim_t d, dg_cell_index_t k)
  { 
    int i;
    dg_grid_pos_t kp[dg_MAX_GRID_DIM];
    dg_rank_t kr = dg_cell_rank(k);
    dg_cell_position(d, k, kp);
    fprintf(wr, "%d:(", (int)kr);
    for (i = 0; i < d; i++) 
      { fprintf(wr, "%s%d", (i == 0 ? "" : ","), (int)kp[i]); }
    fprintf(wr, ")");
  }
  
/* VECTOR ALLOCATION */

dg_cell_index_vec_t dg_cell_index_vec_new(unsigned int nel)
  { /* This is not a macro only because gcc does not allow cast of struct: */
    vec_t v = vec_new(nel, sizeof(dg_cell_index_t));
    dg_cell_index_vec_t r;
    r.nel = v.nel; r.el = (dg_cell_index_t *)v.el;
    return r;
  }
