/* See dg_pulse_N.h */
/* Last edited on 2005-04-21 18:27:42 by stolfi */

#include <dg_grid.h>
#include <dg_pulse.h>
#include <dg_pulse_N.h>
#include <bz_basic.h>
#include <affirm.h>
#include <math.h>

/* INTERNAL PROTOTYPES */

void dg_pulse_N_mother_main_bezier
  ( dg_cont_t c, 
    dg_degree_t g, 
    dg_pulse_mother_index_t m,
    double *bz
  );
  
/* IMPLEMENTATIONS */

unsigned int dg_pulse_N_num_mothers (dg_cont_t c, dg_degree_t g)
  { affirm(g >= 0, "negative degree");
    affirm(c >= -1, "bad continuity class");
    if (g <= 2*c)
      { return 0; }
    else
      { return g-c; }
  }

dg_grid_size_t dg_pulse_N_mother_supp_count
  ( dg_cont_t c, 
    dg_degree_t g, 
    dg_pulse_mother_index_t m
  )
  { affirm(c >= -1, "bad contin for N-pulse");
    affirm(g >= 0, "negative degree for N-pulse");
    affirm(g >= 2*c + 1, "bad degree for N-pulse");
    affirm((m >= 0) && (m <= g-c-1), "invalid index");
    return (m > c ? 1 : 2);
  }

void dg_pulse_N_mother_to_bezier
  ( dg_cont_t c, 
    dg_degree_t g, 
    dg_pulse_mother_index_t m,
    dg_grid_pos_t x,
    double *bz
  )
  { affirm(c >= -1, "bad contin for N-pulse");
    affirm(g >= 0, "negative degree for N-pulse");
    affirm(g >= 2*c + 1, "bad degree for N-pulse");
    affirm(g <= dg_PULSE_MAX_DEGREE, "degree too high for N-pulse");
    affirm((m >= 0) && (m <= g-c-1), "invalid index");
    int j;
    for (j = 0; j <= g; j++) { bz[j] = 0; }
    if (m > c)
      { /* A Bernstein-Bzier element with single-cell support: */
        if (x == 0) { bz[m] = 1; }
      }
    else
      { /* A two-interval pulse: */
        int r, s;
        if (x == 0)
          { int xip = g-c+m;
            bz[xip] = 1.0;
            for (r = xip+1; r <= g; r++) { bz[r] = bz[r-1]/2; }
            for (s = m; s > 0; s--)
              { bz[xip] /= 2;
                for (r = xip+1; r <= g; r++) { bz[r] = (bz[r] + bz[r-1])/2; }
              }
          }
        else if (x == 1) 
          { bz[m] = 1.0;
            for (r = m-1; r >= 0; r--) { bz[r] = bz[r+1]/2; }
            for (s = c-m; s > 0; s--)
              { bz[m] = bz[m]/2;
                for (r = m-1; r >= 0; r--) { bz[r] = (bz[r] + bz[r+1])/2; }
              }
          }
     }
  }
  
/* SPECIALIZED PROCS FOR MAIN PATCH: */

void dg_pulse_N_mother_main_bezier
  ( dg_cont_t c, 
    dg_degree_t g, 
    dg_pulse_mother_index_t m,
    double *bz
  )
  { int r, s;
    bz[m] = 1.0;
    for (r = m-1; r >= 0; r--) { bz[r] = bz[r+1]/2; }
    for (s = m+1; s <= c; s++)
      { bz[m] = bz[m]/2;
        for (r = m-1; r >= 0; r--) { bz[r] = (bz[r] + bz[r+1])/2; }
      }
  }
