/* See dg_tent.h */
/* Last edited on 2005-06-05 15:45:16 by stolfi */

#include <dg_grid.h>
#include <dg_pulse.h>
#include <dg_tent.h>
#include <bz_basic.h>
/* #include <bz_patch.h> */
#include <affirm.h>
#include <math.h>
#include <assert.h>

/* IMPLEMENTATIONS */

void dg_tent_eval
  ( dg_dim_t d, 
    dg_pulse_family_t fam[], 
    dg_pulse_t p[],
    interval_t R[],
    double x[],
    dg_degree_t ord[],
    double f[]
  )
  { double pf[dg_PULSE_MAX_DEGREE + 1];
    f[0] = 1.0;
    int nf = 1;
    int i;
    for (i = 0; i < d; i++) 
      { interval_t *Ri = (R == NULL ? NULL : &(R[i]));
        dg_pulse_eval(&(fam[i]), &(p[i]), Ri, x[i], ord[i], pf);
        int j;
        for (j = 0; j < nf; j++)
          { int k; 
            for (k = 0; k <= ord[i]; k++)
              { f[k*nf + j] = f[j]*pf[k]; }
          }
        nf *= (ord[i] + 1);
      }
  }

void dg_tent_eval_total
  ( dg_dim_t d, 
    dg_pulse_family_t fam[], 
    dg_pulse_t p[],
    interval_t R[],
    double x[],
    dg_degree_t ord,
    double f[]
  )
  { double pf[dg_PULSE_MAX_DEGREE + 1];
    f[0] = 1.0;
    int i;
    for (i = 0; i < d; i++) 
      { interval_t *Ri = (R == NULL ? NULL : &(R[i]));
        dg_pulse_eval(&(fam[i]), &(p[i]), Ri, x[i], ord, pf);
        affirm(FALSE, "not implemented");
      }
  }

dg_tent_vec_t dg_tents_from_cells
  ( dg_dim_t d, 
    dg_tent_mother_index_t tm,
    dg_cell_index_vec_t cell
  )
  { dg_tent_vec_t tv = dg_tent_vec_new(cell.nel);
    int nt = 0;
    int i; 
    for (i = 0; i < cell.nel; i++)
      { dg_cell_index_t ck = cell.el[i];
        tv.el[nt] = (dg_tent_t){tm,ck};
        nt++;
      }
    assert(nt == tv.nel);
    return tv;
  }

/* PRINTOUT: */

void dg_tent_print
  ( FILE *wr, 
    dg_dim_t d,
    dg_pulse_family_t fam[], 
    dg_pulse_t p[]
  )
  { int i;
    /* Print pulse indices: */
    for (i = 0; i < d; i++) 
      { fprintf(wr, "[");
        dg_pulse_print(wr, &(fam[i]), &(p[i]));
        fprintf(wr, "]");
      }
  }

void dg_tent_packed_print
  ( FILE *wr, 
    dg_dim_t d,
    dg_pulse_family_t fam[], 
    dg_tent_t *t
  )
  { /* Unpack puse indices from tent: */
    dg_pulse_t p[d];
    dg_tent_unpack(d, fam, t, p);
    dg_tent_print(wr, d, fam, p);
  }

/* Manipulation of tent vectors: */

dg_tent_vec_t dg_tent_vec_new(unsigned int nel)
  { /* This is not a macro only because gcc does not allow cast of struct: */
    vec_t v = vec_new(nel, sizeof(dg_tent_t));
    dg_tent_vec_t r;
    r.nel = v.nel; r.el = (dg_tent_t *)v.el;
    return r;
  }

